# Copyright 2019 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import urllib
from csv import DictReader, excel_tab
from ftplib import FTP
import re
from urllib import request

from cached_property import cached_property
from retry import retry

from ebi_eva_common_pyutils.command_utils import run_command_with_output
from ebi_eva_common_pyutils.logger import AppLogger


class NCBIAssembly(AppLogger):
    """
    Class that represent an assembly that would originate from NCBI data
    It takes a GCA or GCF accession and can download the assembly report and genomics fasta.
    Using species_scientific_name and assembly_accession it create a directory structure in the provided
    reference_directory:
        - species_scientific_name1
            - assembly_accession1
            - assembly_accession2
        - species_scientific_name2
    the eutils_api_key is only used to retrieve additional contigs if required.
    """

    def __init__(self, assembly_accession, species_scientific_name, reference_directory, eutils_api_key=None):
        self.check_assembly_accession_format(assembly_accession)
        self.assembly_accession = assembly_accession
        self.species_scientific_name = species_scientific_name
        self.reference_directory = reference_directory
        self.eutils_api_key = eutils_api_key

    @staticmethod
    def is_assembly_accession_format(assembly_accession):
        if re.match(r"^GC[F|A]_\d+\.\d+$", assembly_accession) is not None:
            return True
        return False

    @staticmethod
    def check_assembly_accession_format(assembly_accession):
        if not NCBIAssembly.is_assembly_accession_format(assembly_accession):
            raise ValueError(f'Invalid assembly accession: {assembly_accession} it has to be in the form of '
                            'GCF_XXXXXXXXX.X or GCA_XXXXXXXXX.X where X is a number')

    @property
    def assembly_directory(self):
        assembly_directory = os.path.join(
            self.reference_directory,  self.species_scientific_name.lower().replace(' ', '_'), self.assembly_accession
        )
        os.makedirs(assembly_directory, exist_ok=True),
        return assembly_directory

    @property
    def assembly_report_path(self):
        return os.path.join(self.assembly_directory, self.assembly_accession + '_assembly_report.txt')

    @property
    def assembly_fasta_path(self):
        return os.path.join(self.assembly_directory, self.assembly_accession + '.fa')

    @property
    def assembly_compressed_fasta_path(self):
        return os.path.join(self.assembly_directory, self.assembly_accession + '.fa.gz')

    @retry(tries=4, delay=2, backoff=1.2, jitter=(1, 3))
    def _download_file(self, destination_file, url):
        self.info('Download assembly file for %s to %s', self.assembly_accession, destination_file)
        request.urlretrieve(url, destination_file)
        request.urlcleanup()

    @cached_property
    def _ncbi_genome_folder_url_and_content(self):
        """
        Internal property that retrieve and store the NCBI ftp url and content of the genome folder.
        """
        ftp = FTP('ftp.ncbi.nlm.nih.gov', timeout=600)
        ftp.login('anonymous', 'anonymous')
        genome_folder = 'genomes/all/' + '/'.join([self.assembly_accession[0:3], self.assembly_accession[4:7],
                                                   self.assembly_accession[7:10],
                                                   self.assembly_accession[10:13]]) + '/'
        ftp.cwd(genome_folder)
        all_genome_subfolders = []
        ftp.retrlines('NLST', lambda line: all_genome_subfolders.append(line))

        genome_subfolders = [folder for folder in all_genome_subfolders if folder == self.assembly_accession]
        if len(genome_subfolders) != 1:
            self.debug('Cannot find good match for accession folder with "%s": %s match found', self.assembly_accession, len(genome_subfolders))
            genome_subfolders = [folder for folder in all_genome_subfolders if folder.startswith(self.assembly_accession + '_')]
        if len(genome_subfolders) != 1:
            self.debug('Cannot find good match for accession folder with "starting with %s_": %s match found', self.assembly_accession, len(genome_subfolders))
            genome_subfolders = [folder for folder in all_genome_subfolders if folder.startswith(self.assembly_accession)]
        if len(genome_subfolders) != 1:
            self.debug('Cannot find good match for accession folder with "starting with %s": %s match found', self.assembly_accession, len(genome_subfolders))
            genome_subfolders = [folder for folder in all_genome_subfolders if self.assembly_accession in folder]
        if len(genome_subfolders) != 1:
            self.debug('Cannot find good match for accession folder with "%s in name": %s match found', self.assembly_accession, len(genome_subfolders))
            raise Exception('more than one folder matches the assembly accession: ' + str(genome_subfolders))
        ftp.cwd(genome_subfolders[0])
        genome_files = []
        ftp.retrlines('NLST', lambda line: genome_files.append(line))
        url = 'ftp://' + 'ftp.ncbi.nlm.nih.gov' + '/' + genome_folder + genome_subfolders[0]
        ftp.close()
        return url, genome_files

    @cached_property
    def assembly_report_url(self):
        """
        Search on the NCBI FTP for the assembly report file and return the full url if only one found.
        Raise if not.
        """
        url, genome_files = self._ncbi_genome_folder_url_and_content
        assembly_reports = [genome_file for genome_file in genome_files if 'assembly_report.txt' in genome_file]
        if len(assembly_reports) != 1:
            raise Exception('more than one file has "assembly_report" in its name: ' + str(assembly_reports))
        return url + '/' + assembly_reports[0]

    @cached_property
    def assembly_fasta_url(self):
        """
        Search on the NCBI FTP for the assembly genomics fasta file and return the full url if only one found.
        Raise if not.
        """
        url, genome_files = self._ncbi_genome_folder_url_and_content
        assembly_fasta = [genome_file for genome_file in genome_files if genome_file.endswith('_genomic.fna.gz')]
        # Remove the entries that are from genomics dna but the whole genome
        assembly_fasta = [fasta for fasta in assembly_fasta if '_from_' not in fasta]
        if len(assembly_fasta) > 1:
            raise Exception('{} file found ending with "_genomic.fna.gz" in its name: {}'.format(len(assembly_fasta),
                                                                                                 assembly_fasta))
        return url + '/' + assembly_fasta[0]

    def get_assembly_report_rows(self):
        """Download the assembly report if it does not exist then parse it to create a generator
        that return each row as a dict."""
        self.download_assembly_report()
        with open(self.assembly_report_path) as open_file:
            headers = None
            # Parse the assembly report file to find the header then stop
            for line in open_file:
                if line.lower().startswith("# sequence-name") and "sequence-role" in line.lower():
                    headers = line.strip().split('\t')
                    break
            reader = DictReader(open_file, fieldnames=headers, dialect=excel_tab)
            for record in reader:
                yield record

    def download_assembly_report(self, overwrite=False):
        if not os.path.isfile(self.assembly_report_path) or overwrite:
            self._download_file(self.assembly_report_path, self.assembly_report_url)

    def download_assembly_fasta(self, overwrite=False):
        if not os.path.isfile(self.assembly_fasta_path) or overwrite:
            self._download_file(self.assembly_compressed_fasta_path, self.assembly_fasta_url)
            run_command_with_output(
                'Uncompress {}'.format(self.assembly_compressed_fasta_path),
                'gunzip -f {}'.format(self.assembly_compressed_fasta_path)
            )

    def construct_fasta_from_report(self, genbank_only=False):
        """
        Download the assembly report if it does not exist then create the assembly fasta from the contig.
        If the assembly already exist then it only add any missing contig.
        """
        written_contigs = self.get_written_contigs(self.assembly_fasta_path)
        contig_to_append = []
        for row in self.get_assembly_report_rows():
            genbank_accession = row['GenBank-Accn']
            refseq_accession = row['RefSeq-Accn']
            relationship = row['Relationship']
            accession = genbank_accession
            if relationship != '=' and genbank_accession == 'na' and not genbank_only:
                accession = refseq_accession
            if accession in written_contigs:
                self.debug('Accession ' + accession + ' already in the FASTA file, don\'t need to be downloaded')
                continue
            if not accession or accession == 'na':
                raise ValueError('Accession {} found in report is not valid'.format(accession))
            contig_to_append.append(self.download_contig_sequence_from_ncbi(accession))

        # Now append all the new contigs to the existing fasta
        with open(self.assembly_fasta_path, 'a+') as fasta:
            for contig_path in contig_to_append:
                with open(contig_path) as sequence:
                    for line in sequence:
                        # Check that the line is not empty
                        if line.strip():
                            fasta.write(line)
                os.remove(contig_path)

    def download_contig_sequence_from_ncbi(self, accession):
        sequence_tmp_path = os.path.join(self.assembly_directory, accession + '.fa')
        self.download_contig_from_ncbi(accession, sequence_tmp_path)
        self.info(accession + " downloaded and added to FASTA sequence")
        return sequence_tmp_path

    @staticmethod
    def get_written_contigs(fasta_path):
        written_contigs = []
        match = re.compile(r'>(.*?)\s')
        if os.path.isfile(fasta_path):
            with open(fasta_path, 'r') as file:
                for line in file:
                    written_contigs.extend(match.findall(line))
        return written_contigs

    @retry(tries=4, delay=2, backoff=1.2, jitter=(1, 3))
    def download_contig_from_ncbi(self, contig_accession, output_file):
        parameters = {
            'db': 'nuccore',
            'id': contig_accession,
            'rettype': 'fasta',
            'retmode': 'text',
            'tool': 'eva',
            'email': 'eva-dev@ebi.ac.uk'
        }
        if self.eutils_api_key:
            parameters['api_key'] = self.eutils_api_key

        url = 'https://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?' + urllib.parse.urlencode(parameters)
        self.info('Downloading ' + contig_accession)
        urllib.request.urlretrieve(url, output_file)

    def download_or_construct(self, genbank_only=False, overwrite=False):
        """First download the assembly report and fasta from the FTP, then append any missing contig from
        the assembly report to the assembly fasta."""
        self.download_assembly_report(overwrite)
        try:
            self.download_assembly_fasta(overwrite)
        except:
            pass
        # This will either confirm the presence of all the contig or download any one missing
        self.construct_fasta_from_report(genbank_only)



# Copyright 2021 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Rationale for a Nextflow pipeline abstraction
# ---------------------------------------------
# Dynamic pipeline generation
# Abstraction to represent process dependencies
# Unit testability of individual steps without scattering logic between Python and Nextflow
# Ability to combine pipelines

import networkx as nx
import os
from typing import List, Dict, Union

from ebi_eva_common_pyutils.logger import AppLogger
from ebi_eva_common_pyutils.command_utils import run_command_with_output


class NextFlowProcess:

    def __init__(self, process_name: str, command_to_run: str, process_directives: Dict[str, str] = None) -> None:
        """
        Create a Nextflow process
        :rtype: None
        :param process_name: Name of the process - should be a valid identifier - ex: p1_merge
        :type process_name: str
        :param command_to_run: Command to be run - ex: bash -c "echo p1"
        :type command_to_run: str
        :param process_directives: Additional process directives - ex: {"memory": "4GB", "executor": "lsf"}
        :type process_directives: dict
        """
        if not process_name.isidentifier():
            raise ValueError(f"{process_name} is not a valid Nextflow process name")
        self.process_name = process_name
        self.success_flag = f"{self.process_name}_success"
        self.command_to_run = command_to_run
        self.process_directives = process_directives if process_directives else dict()


class NextFlowPipeline(AppLogger):
    def __init__(self, process_dependency_map: Dict[NextFlowProcess, List[NextFlowProcess]] = None) -> None:
        """
        Create a Nextflow pipeline with a process dependency map

        :param process_dependency_map: Map of Nextflow processes and their corresponding dependencies
        - ex: {p3 : [p2], p2: [p1]}  where p1, p2 and p3 are Nextflow processes that should be executed sequentially
        """
        # Modeling the dependency map as a DiGraph (Directed graph) is advantageous
        # in ordering/combining flows and detecting cycles
        self.process_dependency_map = nx.ordered.DiGraph()
        if process_dependency_map:
            self.add_dependencies(process_dependency_map)

    def add_dependencies(self, process_dependency_map: Dict[NextFlowProcess, List[NextFlowProcess]]):
        for process, dependencies in process_dependency_map.items():
            if dependencies:
                for dependency in dependencies:
                    self.add_process_dependency(process, dependency)
            else:
                self.add_process_dependency(process, None)

    def add_process_dependency(self, process: NextFlowProcess, dependency: Union[NextFlowProcess, None]):
        if dependency:
            self.process_dependency_map.add_edge(process, dependency)
            if not nx.dag.is_directed_acyclic_graph(self.process_dependency_map):
                raise ValueError(f"Cycles found in pipeline when adding process {process.process_name} "
                                 f"and its dependency {dependency.process_name}")
        else:
            # If no dependency is specified, the process will just be a single node in the DAG
            self.process_dependency_map.add_node(process)

    def _write_to_pipeline_file(self, workflow_file_path: str):
        with open(workflow_file_path, "a") as pipeline_file_handle:
            pipeline_file_handle.write(self.__str__() + "\n")

    def run_pipeline(self, workflow_file_path: str, nextflow_binary_path: str = 'nextflow',
                     nextflow_config_path: str = None, working_dir: str = ".", resume: bool = False,
                     other_args: dict = None):
        # Remove pipeline file if it already exists
        if os.path.exists(workflow_file_path):
            os.remove(workflow_file_path)
        self._write_to_pipeline_file(workflow_file_path)
        workflow_command = f"cd {working_dir} && {nextflow_binary_path} run {workflow_file_path}"
        workflow_command += f" -c {nextflow_config_path}" if nextflow_config_path else ""
        workflow_command += f" -with-report {workflow_file_path}.report.html"
        workflow_command += f" -with-dag {workflow_file_path}.dag.png"
        workflow_command += " -resume" if resume else ""
        workflow_command += " ".join([f" -{arg} {val}" for arg, val in other_args.items()]) if other_args else ""
        run_command_with_output(f"Running pipeline {workflow_file_path}...", workflow_command)

    @staticmethod
    def join_pipelines(main_pipeline: 'NextFlowPipeline', dependent_pipeline: 'NextFlowPipeline',
                       with_dependencies: bool = True) -> 'NextFlowPipeline':
        """
        Join two pipelines with or without dependencies

        With Dependencies it returns a new pipeline where:
            1) root processes are those of the main pipeline.
            2) final processes are those of the dependent pipeline and
            3) every root process of the dependent pipeline depends on the final processes of the main pipeline.
        Without Dependencies it returns a new pipeline where:
            1) the two pipeline are left independent
            2) Only shared dependencies
            3) every root process of the dependent pipeline depends on the final processes of the main pipeline.

        """
        joined_pipeline = NextFlowPipeline()
        # Aggregate dependency maps of both pipelines
        joined_pipeline.process_dependency_map = nx.compose(main_pipeline.process_dependency_map,
                                                            dependent_pipeline.process_dependency_map)
        if with_dependencies:
            for final_process_in_main_pipeline in main_pipeline._get_final_processes():
                for root_process_in_dependent_pipeline in dependent_pipeline._get_root_processes():
                    joined_pipeline.add_process_dependency(root_process_in_dependent_pipeline,
                                                           final_process_in_main_pipeline)
        return joined_pipeline

    def _get_root_processes(self) -> List[NextFlowProcess]:
        # Root processes are those which have no dependencies
        # See https://stackoverflow.com/a/62948641
        roots = []
        for component in nx.weakly_connected_components(self.process_dependency_map):
            subgraph = self.process_dependency_map.subgraph(component)
            roots.extend([n for n, d in subgraph.out_degree() if d == 0])
        return roots

    def _get_final_processes(self) -> List[NextFlowProcess]:
        # Final processes are those which have no other processes depending on them
        # See https://stackoverflow.com/a/62948641
        roots = []
        for component in nx.weakly_connected_components(self.process_dependency_map):
            subgraph = self.process_dependency_map.subgraph(component)
            roots.extend([n for n, d in subgraph.in_degree() if d == 0])
        return roots

    @staticmethod
    def _get_process_repr(process: NextFlowProcess, dependencies: List[NextFlowProcess]) -> str:
        process_directives_str = "\n".join([f"{key}='{value}'" for key, value in process.process_directives.items()])
        input_dependencies = "val flag from true"
        if dependencies:
            input_dependencies = "\n".join([f"val {dependency.success_flag} from {dependency.success_flag}"
                                            for dependency in dependencies])
        return "\n".join(map(str.strip, f"""
                    process {process.process_name} {{
                    {process_directives_str}
                    input:
                    {input_dependencies}
                    output:
                    val true into {process.success_flag}
                    script:
                    \"\"\"
                    {process.command_to_run}
                    \"\"\"
                    }}""".split("\n")))

    def __str__(self):
        # Order the list of nodes based on the dependency
        # See https://networkx.org/documentation/stable/reference/algorithms/generated/networkx.algorithms.traversal.depth_first_search.dfs_postorder_nodes.html?highlight=dfs_postorder_nodes#networkx.algorithms.traversal.depth_first_search.dfs_postorder_nodes
        ordered_list_of_processes_to_run = list(nx.dfs_postorder_nodes(self.process_dependency_map))
        # Get a Nextflow pipeline representation of each process and its dependencies
        return "\n\n".join([NextFlowPipeline._get_process_repr(process, list(self.process_dependency_map[process]))
                            for process in ordered_list_of_processes_to_run])


class LinearNextFlowPipeline(NextFlowPipeline):
    """
    Simple linear pipeline that supports resumption
    """
    previous_process: NextFlowProcess = None

    def __init__(self, process_list: List[NextFlowProcess] = None):
        dependency_map = {}
        if process_list:
            for index, process in enumerate(process_list):
                dependency_map[process] = [] if index == 0 else [process_list[index - 1]]
        super().__init__(dependency_map)

    def add_process(self, process_name, command_to_run):
        current_process = NextFlowProcess(process_name=process_name, command_to_run=command_to_run)
        self._add_new_process(current_process)

    def _add_new_process(self, current_process):
        super().add_process_dependency(current_process, self.previous_process)
        self.previous_process = current_process



# Copyright 2020 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from urllib.parse import quote_plus
from lxml import etree as et
import json
import yaml
import urllib.request

from pymongo.uri_parser import split_hosts
from retry import retry


class EVAPrivateSettingsXMLConfig:
    config_data = None

    def __init__(self, settings_xml_file: str):
        with open(settings_xml_file) as xml_file_handle:
            self.config_data = et.parse(xml_file_handle)

    def get_value_with_xpath(self, location: str, optional: bool = False):
        etree = self.config_data.getroot()
        result = etree.xpath(location)
        if not result and not optional:
            raise ValueError("Invalid XPath location: " + location)
        return result


def get_metadata_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets host, username, and password for metadata postgres database.
    Useful for filling properties files, for connection purposes you can use
    `metadata_utils.get_metadata_connection_handle`.
    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    pg_url = properties['eva.evapro.jdbc.url']
    pg_user = properties['eva.evapro.user']
    pg_pass = properties['eva.evapro.password']
    return pg_url, pg_user, pg_pass


def get_primary_mongo_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets primary host, username, and password for mongo database.
    Useful for filling properties files, for connection purposes it is preferable to use
    `mongo_utils.get_mongo_connection_handle` as that will handle multiple hosts appropriately.
    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    # Use the primary mongo host from configuration:
    # https://github.com/EBIvariation/configuration/blob/master/eva-maven-settings.xml#L111
    # TODO: revisit once accessioning/variant pipelines can support multiple hosts
    try:
        mongo_host = split_hosts(properties['eva.mongo.host'])[1][0]
    except IndexError:  # some profiles have only one host
        mongo_host = split_hosts(properties['eva.mongo.host'])[0][0]
    mongo_user = properties['eva.mongo.user']
    mongo_pass = properties['eva.mongo.passwd']
    return mongo_host, mongo_user, mongo_pass


def get_accession_pg_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets host, username, and password for accessioning job tracker database.
    Useful for filling properties files.
    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    pg_url = properties['eva.accession.jdbc.url']
    pg_user = properties['eva.accession.user']
    pg_pass = properties['eva.accession.password']
    return pg_url, pg_user, pg_pass


def get_variant_load_job_tracker_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets host, username, and password for variant load job tracker database.
    Useful for filling properties files.
    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    variant_url = properties['eva.variant.jdbc.url']
    variant_user = properties['eva.variant.user']
    variant_pass = properties['eva.variant.password']
    return variant_url, variant_user, variant_pass


def get_contig_alias_db_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets url, username, and password for contig alias database.

    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    contig_alias_url = properties['contig-alias.url']
    contig_alias_user = properties['contig-alias.admin-user']
    contig_alias_pass = properties['contig-alias.admin-password']

    return contig_alias_url, contig_alias_user, contig_alias_pass


def get_count_service_creds_for_profile(profile_name: str, settings_xml_file: str):
    """
    Gets host, username, and password for eva count service.
    Useful for filling properties files.
    """
    properties = get_properties_from_xml_file(profile_name, settings_xml_file)
    counts_url = properties['eva.count-stats.url']
    counts_user = properties['eva.count-stats.username']
    counts_pass = properties['eva.count-stats.password']
    return counts_url, counts_user, counts_pass


def get_pg_uri_for_accession_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.accession.jdbc.url")


def get_pg_uri_for_variant_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.variant.jdbc.url")


def get_pg_metadata_uri_for_eva_profile(profile_name: str, settings_xml_file: str):
    return get_pg_uri_details_for_profile(profile_name, settings_xml_file, "eva.evapro.jdbc.url")


def get_pg_uri_details_for_profile(eva_profile_name: str, settings_xml_file: str, tag_name: str):
    config = EVAPrivateSettingsXMLConfig(settings_xml_file)
    xpath_location_template = '//settings/profiles/profile/id[text()="{0}"]/../properties/{1}/text()'
    # Format is jdbc:postgresql://host:port/db
    metadata_db_jdbc_url = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, tag_name))[0]
    return metadata_db_jdbc_url.split("jdbc:")[-1]


def get_mongo_uri_for_eva_profile(eva_profile_name: str, settings_xml_file: str):
    config = EVAPrivateSettingsXMLConfig(settings_xml_file)
    xpath_location_template = '//settings/profiles/profile/id[text()="{0}"]/../properties/{1}/text()'
    # Format is host1:port1,host2:port2
    mongo_hosts_and_ports = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.host"))[0]
    username = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.user"), optional=True)
    if not username:  # no authentication
        return f"mongodb://{mongo_hosts_and_ports}"
    username = username[0]
    password = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.passwd"))[0]
    authentication_db = config.get_value_with_xpath(
        xpath_location_template.format(eva_profile_name, "eva.mongo.auth.db"))[0]
    return "mongodb://{0}:{1}@{2}/{3}".format(username, quote_plus(password), mongo_hosts_and_ports, authentication_db)


def get_properties_from_xml_file(profile, xml_path):
    tree = et.parse(xml_path)
    root = tree.getroot()
    return get_profile_properties(profile, root)


def get_properties_from_xml_string(profile, str):
    root = et.fromstring(str)
    return get_profile_properties(profile, root)


def get_profile_properties(profile, root):
    properties = {}
    for property in root.xpath('//settings/profiles/profile/id[text()="' + profile + '"]/../properties/*'):
        properties[property.tag] = property.text
    return properties


@retry(tries=4, delay=2, backoff=1.2, jitter=(1, 3))
def get_eva_settings_xml_string(token):
    url = 'https://api.github.com/repos/EBIvariation/configuration/contents/eva-maven-settings.xml'
    headers = {'Authorization': 'token ' + token, 'Accept' : 'application/vnd.github.raw' }
    request = urllib.request.Request(url, None, headers)
    with urllib.request.urlopen(request) as response:
        return response.read()


def get_args_from_private_config_file(private_config_file):
    with open(private_config_file) as private_config_file_handle:
        if 'json' in private_config_file:
            return json.load(private_config_file_handle)
        else:
            if 'yml' in private_config_file:
                return yaml.safe_load(private_config_file_handle)
            else:
                raise TypeError('Configuration file should be either json or yaml')


# Copyright 2019 EMBL - European Bioinformatics Institute
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from ebi_eva_common_pyutils.logger import logging_config
from ebi_eva_common_pyutils.network_utils import json_request
from ebi_eva_common_pyutils.reference.assembly import NCBIAssembly
from ebi_eva_common_pyutils.taxonomy.taxonomy import get_scientific_name_from_ensembl

logging_config.add_stdout_handler()
logger = logging_config.get_logger(__name__)


def get_supported_asm_from_ensembl(tax_id: int) -> str:
    logger.info(f'Query Ensembl for species name using taxonomy {tax_id}')
    scientific_name_api_param = get_scientific_name_from_ensembl(tax_id).lower().replace(' ', '_')
    ENSEMBL_REST_API_URL = "http://rest.ensembl.org/info/assembly/{0}?content-type=application/json".format(
        scientific_name_api_param)
    response = json_request(ENSEMBL_REST_API_URL)
    assembly_accession_attribute = 'assembly_accession'
    if assembly_accession_attribute in response:
        return str(response.get(assembly_accession_attribute))
    return None



import os
from distutils.core import setup

from setuptools import find_packages

setup(
    name='ebi_eva_common_pyutils',
    scripts=[os.path.join(os.path.dirname(__file__), 'ebi_eva_common_pyutils', 'archive_directory.py')],
    packages=find_packages(),
    version='0.5.3',
    license='Apache',
    description='EBI EVA - Common Python Utilities',
    url='https://github.com/EBIVariation/eva-common-pyutils',
    keywords=['EBI', 'EVA', 'PYTHON', 'UTILITIES'],
    install_requires=['psycopg2-binary', 'requests', 'pymongo', 'lxml', 'pyyaml', 'cached-property', 'retry',
                      'networkx<=2.5'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 3'
    ]
)
